@@ -17,7 +17,7 @@ except ImportError:
 
 from flask import Flask, jsonify, render_template, request, url_for
 from libearth.codecs import Rfc3339
-from libearth.compat import binary
+from libearth.compat import binary, text_type
 from libearth.crawler import CrawlError, crawl
 from libearth.parser.autodiscovery import autodiscovery, FeedUrlNotFoundError
 from libearth.repository import FileSystemRepository, from_url
@@ -60,23 +60,20 @@ def crawl_category():
             crawling_queue.task_done()
         elif priority == 1:
             cursor, feed_id = arguments
-            ids = {}
+            urls = []
 
             if not feed_id:
-                for sub in cursor.recursive_subscriptions:
-                    ids[sub.feed_uri] = sub.feed_id
+                urls = [sub.feed_uri for sub in cursor.recursive_subscriptions]
             else:
-                for sub in cursor.recursive_subscriptions:
-                    if sub.feed_id == feed_id:
-                        ids[sub.feed_uri] = sub.feed_id
-                        break
+                urls = [sub.feed_uri for sub in cursor.recursive_subscriptions
+                        if sub.feed_id == feed_id]
 
-            iterator = iter(crawl(ids.keys(), app.config['CRAWLER_THREAD']))
+            iterator = iter(crawl(urls, app.config['CRAWLER_THREAD']))
             while True:
                 try:
                     feed_url, feed_data, crawler_hints = next(iterator)
                     with get_stage() as stage:
-                        stage.feeds[ids[feed_url]] = feed_data
+                        stage.feeds[feed_data.feed_id] = feed_data
                 except CrawlError:
                     continue
                 except StopIteration:
@@ -218,7 +215,7 @@ def get_stage():
 
         session = Session(session_id)
         url = urlparse.urlparse(app.config['REPOSITORY'])
-        if url.scheme == 'file' or url.scheme == '':
+        if url.scheme == 'file':
             repository = FileSystemRepository(
                 url.path,
                 atomic=request.environ['wsgi.multithread']
@@ -333,8 +330,7 @@ def delete_feed(category_id, feed_id):
     target = None
     for subscription in cursor:
         if isinstance(subscription, Subscription):
-            if feed_id == hashlib.sha1(
-                    binary(subscription.feed_uri)).hexdigest():
+            if feed_id == subscription.feed_id:
                 target = subscription
     if target:
         cursor.discard(target)
@@ -504,7 +500,7 @@ class FeedEntryGenerator():
             raise StopIteration
         entry_permalink = get_permalink(self.entry)
         entry_data = {
-            'title': clean_html(str(self.entry.title)),
+            'title': clean_html(text_type(self.entry.title)),
             'entry_id': get_hash(self.entry.id),
             'permalink': entry_permalink or None,
             'updated': Rfc3339().encode(self.entry.updated_at.astimezone(utc)),
@@ -557,10 +553,10 @@ def feed_entries(category_id, feed_id):
         except IteratorNotFound:
             pass
     else:
-        url_token = str(now())
+        url_token = text_type(now())
     if not generator:
         it = iter(feed.entries)
-        feed_title = clean_html(str(feed.title))
+        feed_title = clean_html(text_type(feed.title))
         feed_permalink = get_permalink(feed)
         generator = FeedEntryGenerator(category_id, feed_id, feed_title,
                                        feed_permalink, it, now(), read, starred)
@@ -589,7 +585,7 @@ def feed_entries(category_id, feed_id):
             feed_id
         )
     return jsonify(
-        title=clean_html(str(feed.title)),
+        title=clean_html(text_type(feed.title)),
         entries=entries,
         next_url=next_url
     )
@@ -677,7 +673,7 @@ def category_entries(category_id):
         except IteratorNotFound:
             pass
     else:
-        url_token = str(now())
+        url_token = text_type(now())
     if not generator:
         subscriptions = cursor.recursive_subscriptions
         generator = CategoryEntryGenerator()
@@ -692,12 +688,12 @@ def category_entries(category_id):
                     feed = stage.feeds[subscription.feed_id]
             except KeyError:
                 continue
-            feed_id = get_hash(feed.id)
-            feed_title = clean_html(str(feed.title))
+            feed_title = clean_html(text_type(feed.title))
             it = iter(feed.entries)
             feed_permalink = get_permalink(feed)
-            child = FeedEntryGenerator(category_id, feed_id, feed_title,
-                                       feed_permalink, it, now(), read, starred)
+            child = FeedEntryGenerator(category_id, subscription.feed_id,
+                                       feed_title, feed_permalink, it, now(),
+                                       read, starred)
             generator.add(child)
         generator.set_generators(id_after, time_after)
     save_entry_generators(url_token, generator)
@@ -774,13 +770,13 @@ def feed_entry(category_id, feed_id, entry_id):
         content = content.sanitized_html
 
     entry_data = {
-        'title': clean_html(str(entry.title)),
+        'title': clean_html(text_type(entry.title)),
         'content': content,
-        'updated': entry.updated_at.__str__(),
+        'updated': text_type(entry.updated_at),
         'permalink': entry_permalink or None,
     }
     feed_data = {
-        'title': clean_html(str(feed.title)),
+        'title': clean_html(text_type(feed.title)),
         'permalink': feed_permalink or None
     }
     add_urls(
