@@ -35,10 +35,7 @@ function changeFilter(event) {
 }
 
 function clickComplementaryMenu(event) {
-	var target = $(event.target);
-	while (target.attr('data-action') === undefined) {
-		target = target.parent();
-	}
+	var target = $(event.target).closest('[data-action]');
 
 	var action = target.attr('data-action');
 	closeSide();
@@ -64,13 +61,7 @@ function removeCurrentSelected() {
 	}
 	url = current.attr('data-remove-feed-url') || current.attr('data-remove-category-url');
 	if (url) {
-		var parentMenu = current;
-		while (parentMenu.hasClass('fold') === false) {
-			parentMenu = parentMenu.parent();
-			if (parentMenu.length === 0) {
-				break;
-			}
-		}
+		var parentMenu = current.closest('.fold');
 
 		if (confirm('remove ' + current.text() + '\nAre you sure?') === true){
 			$.ajax({
@@ -78,6 +69,9 @@ function removeCurrentSelected() {
 				'type': 'delete',
 			}).done(function(obj) {
 				makeFeedList(obj, parentMenu);
+			}).fail(function(xhr, status, err) {
+				var json = xhr.responseJSON;
+				alert(json.error + "\n" + json.message);
 			});
 		}
 	}
@@ -139,13 +133,7 @@ function toggleSide(event) {
 }
 
 function toggleFolding(event) {
-	var target = $(event.target);
-	while (target.hasClass('header') === false) {
-		target = target.parent();
-		if (target === null) {
-			return;
-		}
-	}
+	var target = $(event.target).closest('.header');
 
 	target.toggleClass('closed');
 }
@@ -179,6 +167,9 @@ function processForm(event) {
 	} catch (err) {
 	}
 
+	var submit = target.find('input[type=submit]');
+	submit.prop('disabled', true);
+
 	$.post(action,data).done(function(res) {
 		if (current === null) {
 			makeFeedList(res);
@@ -193,6 +184,8 @@ function processForm(event) {
 	}).fail(function(xhr, status, err) {
 		var json = xhr.responseJSON;
 		alert(json.error + "\n" + json.message);
+	}).always(function() {
+		submit.prop('disabled', false);
 	});
 }
 
@@ -518,13 +511,9 @@ function clickFeed(event) {
 }
 
 function clickEntry(event) {
-	var target = $(event.target);
+	var target = $(event.target).closest('.entry-title');
 	var main = $('[role=main]');
 
-	while (target.hasClass('entry-title') === false) {
-		target = target.parent();
-	}
-
 	var entry = target.parent();
 	var entry_url = entry.attr('data-entries');
 
@@ -676,7 +665,17 @@ function keyboardShortcut(event) {
 		}
 	};
 
+	var closeEntry = function closeEntry() {
+		main.find('article.current .entry-content').remove();
+	};
+
 	switch (event.keyCode) {
+		case 13: // return
+			closeEntry();
+			break;
+		case 27: // esc
+			closeManual();
+			break;
 		case 74: //j
 			nextEntry();
 			break;
